@@ -1,6 +1,6 @@
 import struct
 import logging
-from e6xdb.date_time_utils import FORMATS
+from e6xdb.date_time_utils import FORMATS, floor_div, floor_mod
 from datetime import datetime, timedelta
 from e6xdb.constants import ZONE
 
@@ -103,13 +103,6 @@ def read_values_from_array(query_columns_description: list, dis: DataInputStream
         dtype = i.get_field_type()
         isPresent = dis.read_byte()
         date_format = i.get_format()
-        if date_format is None:
-            date_format = "%Y-%m-%d %H:%M:%S"
-        elif len(date_format) == 0:
-            date_format = "%Y-%m-%d %H:%M:%S"
-        else:
-            for dateformat in FORMATS:
-                date_format = date_format.replace(dateformat, FORMATS[dateformat])
         if isPresent == 0:
             value_array.append(None)
             continue
@@ -117,13 +110,16 @@ def read_values_from_array(query_columns_description: list, dis: DataInputStream
             if dtype == "LONG":
                 value_array.append(dis.read_long())
             elif dtype == "DATE":
-                epoch_days = dis.read_long() * 86400
-                date = datetime.fromtimestamp(epoch_days)
-                value_array.append(date.strftime(date_format))
+                epoch_seconds = floor_div(dis.read_long(), 1000_000)
+                date = datetime.fromtimestamp(epoch_seconds, ZONE)
+                value_array.append(date.strftime("%Y-%m-%d"))
             elif dtype == "DATETIME":
-                epoch_days = dis.read_long()
-                date_time = datetime.fromtimestamp(epoch_days, ZONE)
-                value_array.append(date_time.strftime(date_format))
+                epoch_micros = dis.read_long()
+                epoch_seconds = floor_div(epoch_micros, 1000_000)
+                micros_of_the_day = floor_mod(epoch_micros, 1000_000)
+                date_time = datetime.fromtimestamp(epoch_seconds, ZONE)
+                date_time = date_time + timedelta(microseconds=micros_of_the_day)
+                value_array.append(date_time.strftime("%Y-%m-%d %H:%M:%S"))
             elif dtype == "STRING" or dtype == "ARRAY" or dtype == "MAP" or dtype == "STRUCT":
                 value_array.append(dis.read_utf().decode())
             elif dtype == "INT":
@@ -147,7 +143,7 @@ def read_values_from_array(query_columns_description: list, dis: DataInputStream
                 time = dis.read_long()
                 date_time = datetime.fromtimestamp((julian_day - 2440588) * 86400)
                 date_time_with_nanos = date_time + timedelta(microseconds=(time / 1000))
-                value_array.append(date_time_with_nanos.strftime("%Y-%m-%d %H:%M:%S"))
+                value_array.append(date_time_with_nanos)
             elif dtype == "INTEGER":
                 value_array.append(dis.read_int())
         except Exception as e:
